unit MainFormU;

interface

uses
  Androidapi.Jni.JavaTypes,
  Androidapi.JniBridge,
  System.Messaging,
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs,
  FMX.Controls.Presentation, FMX.StdCtrls, FMX.Maps;

type
  TMainForm = class(TForm)
    Button1: TButton;
    MapPanel: TPanel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1Click(Sender: TObject);
  private
    FMapView: TMapView;
    procedure RequestPermissions;
    procedure HandlePermissionsRequest(const Sender: TObject; const M: TMessage);
    procedure OnPermissionsRequest(ARequestCode: Integer;
      APermissions: TJavaObjectArray<JString>; AGrantResults: TJavaArray<Integer>);
    procedure SetupMap(ALocationPermissionGranted: Boolean);
  public
    { Public declarations }
  end;

var
  MainForm: TMainForm;

implementation

{$INLINE AUTO}

uses
  Androidapi.Jni,
  Androidapi.Jni.Os,
  Androidapi.Jni.GraphicsContentViewText,
  Androidapi.Jni.Widget,
  Androidapi.Helpers,
  FMX.Helpers.Android,
  FMX.DialogService.Async;

{$R *.fmx}

const
  MY_PERMISSIONS_REQUEST_ACCESS_LOCATION = 1;

procedure Toast(const Msg: string; Duration: Integer);
begin
  CallInUiThread(
    procedure
    begin
      TJToast.JavaClass.makeText(
        TAndroidHelper.Context,
        StrToJCharSequence(msg),
        Duration).show
    end);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  TMessageManager.DefaultManager.SubscribeToMessage(
    TPermissionsRequestResultMessage, HandlePermissionsRequest);
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  TMessageManager.DefaultManager.Unsubscribe(TPermissionsRequestResultMessage, HandlePermissionsRequest);
end;

procedure TMainForm.Button1Click(Sender: TObject);
begin
  // API 23+ calls guarded by a runtime check against the OS SDK version
  if (TJBuild_VERSION.JavaClass.SDK_INT >= 23) then
  begin
    if (TAndroidHelper.Activity.checkSelfPermission(TJManifest_permission.JavaClass.ACCESS_COARSE_LOCATION) <> TJPackageManager.JavaClass.PERMISSION_GRANTED) and
       (TAndroidHelper.Activity.checkSelfPermission(TJManifest_permission.JavaClass.ACCESS_FINE_LOCATION) <> TJPackageManager.JavaClass.PERMISSION_GRANTED) then
      RequestPermissions
    else // Permission has already been granted
      SetupMap(True);
  end
  else
    SetupMap(True);
end;

procedure TMainForm.RequestPermissions;
begin
  var Perms := TJavaObjectArray<JString>.Create(2);
  Perms[0] := TJManifest_permission.JavaClass.ACCESS_COARSE_LOCATION;
  Perms[1] := TJManifest_permission.JavaClass.ACCESS_FINE_LOCATION;
  // Since this whole procedure is only called in an API-guarded manner (from Button1Click),
  // there is no need to check the OS version here
  TAndroidHelper.Activity.requestPermissions(Perms, MY_PERMISSIONS_REQUEST_ACCESS_LOCATION);
  // MY_PERMISSIONS_REQUEST_ACCESS_LOCATION is an app-defined int constant.
  // The callback method, OnPermissionsRequest, gets the result of the request.
end;

procedure TMainForm.HandlePermissionsRequest(const Sender: TObject; const M: TMessage);
begin
  if M is TPermissionsRequestResultMessage then
  begin
    var MessageData := TPermissionsRequestResultMessage(M).Value;
    var RequestCode := MessageData.RequestCode;
    var Permissions := MessageData.Permissions;
    var GrantResults := MessageData.GrantResults;
    OnPermissionsRequest(RequestCode, Permissions, GrantResults);
  end;
end;

procedure TMainForm.OnPermissionsRequest(ARequestCode: Integer; APermissions: TJavaObjectArray<JString>; AGrantResults: TJavaArray<Integer>);
begin
  if ARequestCode = MY_PERMISSIONS_REQUEST_ACCESS_LOCATION then
  begin
    var LocationPermissionGranted :=
      (AGrantResults.Length = 2) and
      (AGrantResults[0] = TJPackageManager.JavaClass.PERMISSION_GRANTED) and
      (AGrantResults[1] = TJPackageManager.JavaClass.PERMISSION_GRANTED);
    if LocationPermissionGranted then
      Toast('User granted permission', TJToast.JavaClass.LENGTH_SHORT)
    else
      Toast('User denied permission', TJToast.JavaClass.LENGTH_SHORT);

    SetupMap(LocationPermissionGranted);
  end;
end;

procedure TMainForm.SetupMap(ALocationPermissionGranted: Boolean);
begin
  // Remember to set up the API key in the Version Info page of the Projct options dialog, as per:
  // http://docwiki.embarcadero.com/RADStudio/en/Configuring_Android_Applications_to_Use_Google_Maps#Configuring_Specific_Project_Options
  if FMapView = nil then
  begin
    FMapView := TMapView.Create(Self);
    FMapView.Align := TAlignLayout.Client;
    FMapView.LayerOptions := [TMapLayerOption.PointsOfInterest, TMapLayerOption.Buildings, TMapLayerOption.Traffic];
    FMapView.ControlOptions := [TMapControlOption.Zoom, TMapControlOption.Compass];
    FMapView.Parent := MapPanel;
  end;
  if ALocationPermissionGranted then
  begin
    // These options only get respected/actioned if the app has one or both access location permissions
    FMapView.LayerOptions := FMapView.LayerOptions + [TMapLayerOption.UserLocation];
    FMapView.ControlOptions := FMapView.ControlOptions + [TMapControlOption.MyLocation];
  end;
end;

end.
